@@ -9,6 +9,7 @@ import (
 	"net/netip"
 	"os"
 	"path/filepath"
+	"runtime"
 	"sync"
 	"testing"
 	"time"
@@ -67,6 +68,32 @@ func TestReaderBytes(t *testing.T) {
 	}
 }
 
+func TestReaderLeaks(t *testing.T) {
+	collected := make(chan struct{})
+
+	func() {
+		r, err := Open(testFile("GeoLite2-City-Test.mmdb"))
+		require.NoError(t, err)
+
+		// We intentionally do NOT call Close() to test if GC picks it up
+		// and if AddCleanup doesn't prevent it.
+
+		runtime.SetFinalizer(r, func(*Reader) {
+			close(collected)
+		})
+	}()
+
+	require.Eventually(t, func() bool {
+		runtime.GC()
+		select {
+		case <-collected:
+			return true
+		default:
+			return false
+		}
+	}, 1*time.Second, 50*time.Millisecond, "Reader was NOT collected (leak detected)")
+}
+
 func TestLookupNetwork(t *testing.T) {
 	bigInt := new(big.Int)
 	bigInt.SetString("1329227995784915872903807060280344576", 10)
@@ -698,16 +725,6 @@ func TestDecodingToNonPointer(t *testing.T) {
 	require.NoError(t, reader.Close(), "error on close")
 }
 
-// func TestNilLookup(t *testing.T) {
-// 	reader, err := Open(testFile("MaxMind-DB-test-decoder.mmdb"))
-// 	require.NoError(t, err)
-
-// 	var recordInterface any
-// 	err = reader.Lookup(nil).Decode( recordInterface)
-// 	assert.Equal(t, "IP passed to Lookup cannot be nil", err.Error())
-// 	require.NoError(t, reader.Close(), "error on close")
-// }
-
 func TestUsingClosedDatabase(t *testing.T) {
 	reader, err := Open(testFile("MaxMind-DB-test-decoder.mmdb"))
 	require.NoError(t, err)
@@ -1173,45 +1190,6 @@ func (c *TestCity) UnmarshalMaxMindDB(d *mmdbdata.Decoder) error {
 	return nil
 }
 
-// TestASN represents ASN data for testing custom unmarshaling.
-type TestASN struct {
-	AutonomousSystemOrganization string `maxminddb:"autonomous_system_organization"`
-	AutonomousSystemNumber       uint   `maxminddb:"autonomous_system_number"`
-}
-
-// UnmarshalMaxMindDB implements the Unmarshaler interface for TestASN.
-func (a *TestASN) UnmarshalMaxMindDB(d *mmdbdata.Decoder) error {
-	mapIter, _, err := d.ReadMap()
-	if err != nil {
-		return err
-	}
-	for key, err := range mapIter {
-		if err != nil {
-			return err
-		}
-
-		switch string(key) {
-		case "autonomous_system_organization":
-			org, err := d.ReadString()
-			if err != nil {
-				return err
-			}
-			a.AutonomousSystemOrganization = org
-		case "autonomous_system_number":
-			asn, err := d.ReadUint32()
-			if err != nil {
-				return err
-			}
-			a.AutonomousSystemNumber = uint(asn)
-		default:
-			if err := d.SkipValue(); err != nil {
-				return err
-			}
-		}
-	}
-	return nil
-}
-
 // TestFallbackToReflection verifies that types without UnmarshalMaxMindDB still work.
 func TestFallbackToReflection(t *testing.T) {
 	reader, err := Open(testFile("MaxMind-DB-test-decoder.mmdb"))
